import {
  Loader, Metadata, Config, Http, IEventParam, IDictionary, I18n,
} from '@zeedhi/core';
import {
  Form, Button, Select, CheckboxMultiple, SpeedDial, DialogService, Checkbox, LoadingService,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme00800CalculoPrevConsController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');

  public blockActivator = false;

  private impConfig = {
    IMPFILTRO: 'N',
    IMPSERV: 'N',
  };

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterAgreg({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnid.value ? agrupUnid.value : 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterSubGrupProd({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORMCALCPREVCONS');
    const row: IDictionary = form.value;
    
    this.setParams(row);
    if (this.utilities.centralIsValid() && this.formIsValid(form)) {
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'Central não informada.', 'error');
      else {
        row.TAMACNIVEL = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
        row.AGRUPNIVEL = row.NIVEL;
        if ((row.CD_REPORT_CODE === 'PME00801' || row.CD_REPORT_CODE === 'PME00803') && row.CDTIPOCUSTO === null) {
          this.utilities.showAlert('alertError', 'Tipo de custo não informado.', 'error');
        } else {
          LoadingService.show(' ');
          const promisses = [];
          if (row.CALCULAR === true) {
            promisses.push(Http.post('calculatePrevCons', { requestType: 'Row', row }));
          }
          promisses.push(Http.post('beforeGenerateReport', { requestType: 'Row', row }));
          Promise.all(promisses).then((results: any[]) => {
            const resultRow = results.find((result) => result.config.url === 'beforeGenerateReport');
            const processedRow = resultRow?.data?.dataset?.row.length > 0 ? resultRow?.data?.dataset?.row[0] : row;
            this.reportController.callGenerateReport(component, processedRow, 'report/calcPrevCons');
            DialogService.hide();
          }).catch((error: any) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          }).finally(() => {
            LoadingService.hide();
          });
        }
      }
    }
  }
  
  public generateOrCalculateReport({ component }: IEventParam<Button>) {
    let nameFunction: () => void;
    const form = Metadata.getInstance<Form>('FORMCALCPREVCONS');
    
    if (component.name === 'calculoPrev') nameFunction = () => this.calculateCons();
    else nameFunction = () => this.generateReport({ component });
    
    const shouldShowDialog = component.name === 'calculoPrev'
      ? form.value.ACRESCIMO < 0
      : form.value.CALCULAR === true && form.value.ACRESCIMO < 0;
    
    if (shouldShowDialog) {
      DialogService.show({
        name: `${component.name}Lancamento`,
        type: 'warning',
        title: 'Atenção',
        text: I18n.translate('pme::confirmarPercentualNegativo'),
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => nameFunction() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else nameFunction();
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    row.IMPFILTRO = this.impConfig.IMPFILTRO;
    row.IMPSERV = this.impConfig.IMPSERV;
    row.CDTIPOCUSTO = row.TIPOCUSTO ? row.TIPOCUSTO.CDTIPOCUSTO : null;
    row.NMTIPOCUSTO = row.TIPOCUSTO ? row.TIPOCUSTO.NMTIPOCUSTO : null;
    row.PRODINI = row.PRODINICIAL ? this.utilities.getValueWithMaskProduto(row.PRODINICIAL.CDARVPROD) : '';
    row.PRODFIN = row.PRODFINAL ? this.utilities.getValueWithMaskProduto(row.PRODFINAL.CDARVPROD) : '';
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.PRODUCTMASK = this.utilities.formatReportMask(mask);
    row.NIVEL = row.NIVEL !== undefined ? row.NIVEL : 0;
    row.ACRESCIMO = row.ACRESCIMO !== undefined ? row.ACRESCIMO : 0;
  }

  public calculateCons() {
    const form = Metadata.getInstance<Form>('FORMCALCPREVCONS');
    const elActions = Metadata.getInstance<SpeedDial>('actionsCalculoPrevConsumo');

    if (this.utilities.centralIsValid() && this.formIsValid(form)) {
      LoadingService.show();
      this.blockActivator = true;
      const row: IDictionary = form.value;
      row.CDFILCENTPME = this.appController.getCdCentral();
      if (!row.CDFILCENTPME) {
        this.utilities.showAlert('alertError', 'Central não informada.', 'error');
        LoadingService.hide();
      } else {
        this.utilities.activeLoadingRoundButton(elActions);
        Http.post('calculatePrevCons', { requestType: 'Row', row }).then(() => {
          this.utilities.showAlert('alertSuccess', 'Calculado com Sucesso.', 'success');
        }).finally(() => {
          this.utilities.disableLoadingRoundButton(elActions);
          this.blockActivator = false;
          LoadingService.hide();
          DialogService.hide();
        });
      }
    }
  }
  
  public activatorClick({ event }: IEventParam<Button>) {
    if (this.blockActivator) event?.stopPropagation();
  }

  private formIsValid(form: IDictionary) {
    let valid = false;
    const row: IDictionary = form.value;
    if (form.validate()) {
      const dtDe = row.DTDE;
      const dtAte = row.DTATE;

      if (this.utilities.dateIsValid(dtDe, dtAte)) {
        const prodIni = row.PRODINICIAL;
        const prodFin = row.PRODFINAL;
        if ((prodIni == null || prodFin == null)
            || this.utilities.prodIsValid(prodIni.CDARVPROD, prodFin.CDARVPROD)
        ) {
          valid = true;
        } else this.utilities.showAlert('alertError', 'Produto inicial maior que o final.', 'error');
      } else this.utilities.showAlert('alertError', 'Data Inicial deve ser menor que a data final.', 'error');
    }
    return valid;
  }

  public setImpConfig({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = component.value.join('_');
      if (items.includes('IMPFILTRO')) this.impConfig.IMPFILTRO = 'S';
      else this.impConfig.IMPFILTRO = 'N';

      if (items.includes('IMPSERV')) this.impConfig.IMPSERV = 'S';
      else this.impConfig.IMPSERV = 'N';
    }
  }

  public clearFields({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const servico = Metadata.getInstance<Select>('SERVICO');

    if (component.name === 'TIPOCARD') {
      servico.setValue('');
      unidade.setValue('');
    } else if (component.name === 'GRUPFILI') unidade.setValue('');
    else if (component.name === 'GRUPSERV') servico.setValue('');
  }
  
  public clearCheckbox() {
    const check = Metadata.getInstance<Checkbox>('CALCULAR');
    check.value = false;
  }
}
